package me.totalfreedom.totalfreedommod.rank;

import me.totalfreedom.totalfreedommod.TotalFreedomMod;
import net.kyori.adventure.text.Component;
import net.luckperms.api.model.data.DataType;
import net.luckperms.api.model.group.Group;
import net.luckperms.api.node.types.PrefixNode;
import net.luckperms.api.node.types.WeightNode;
import net.md_5.bungee.api.ChatColor;
import org.apache.commons.lang.StringUtils;
import org.jetbrains.annotations.NotNull;

import java.util.Locale;
import java.util.concurrent.CompletableFuture;

public class DisplayableGroup implements Displayable
{
    private final Group group;

    private final String name;

    private final String abbr;
    private final String plural;
    private final String article;
    private final int weight;

    private final String tag;

    private final Component coloredTag;

    private final ChatColor color;

    private final org.bukkit.ChatColor teamColor;

    private final boolean hasTeam;

    private final boolean hasDefaultLoginMessage;

    public DisplayableGroup(String group,
                            String plural,
                            String tag,
                            int weight,
                            ChatColor color,
                            org.bukkit.ChatColor teamColor,
                            boolean hasTeam,
                            boolean hasDefaultLoginMessage)
    {
        Group matched = TotalFreedomMod.getPlugin().lpb.getAPI().getGroupManager().getGroup(group);

        if (matched == null) {
            CompletableFuture<Group> cfg = TotalFreedomMod.getPlugin()
                    .lpb
                    .getAPI()
                    .getGroupManager()
                    .createAndLoadGroup(group);

            cfg.thenAcceptAsync(g -> {
                WeightNode weightNode = WeightNode.builder(weight).build();
                PrefixNode prefixNode = PrefixNode.builder().prefix(ChatColor.DARK_GRAY + "[" + color + tag + ChatColor.DARK_GRAY + "]" + getColor()).build();
                g.getData(DataType.NORMAL).add(prefixNode);
                g.getData(DataType.NORMAL).add(weightNode);
            }).join(); // Block until the group is created and loaded.

            matched = TotalFreedomMod.getPlugin().lpb.getAPI().getGroupManager().getGroup(group);

            if (matched == null) throw new IllegalArgumentException("Group " + group + " does not exist and could not be created.");
        }

        this.group = matched;
        this.name = (matched.getDisplayName() != null) ? matched.getDisplayName() : matched.getName();
        this.plural = plural;
        this.article = StringUtils.startsWithAny(this.name.toLowerCase(Locale.ROOT), new String[]{"a", "e", "i", "o", "u"}) ? "an" : "a";
        this.abbr = tag;
        this.weight = weight;
        this.tag = "[" + tag + "]";
        this.color = color;
        this.teamColor = teamColor;
        this.hasTeam = hasTeam;
        this.coloredTag = generateColoredTag(color + tag);
        this.hasDefaultLoginMessage = hasDefaultLoginMessage;
    }

    public boolean isAtLeast(@NotNull DisplayableGroup rank)
    {
        return rank.getLuckPermsGroup().getWeight().orElse(0)
                <= getLuckPermsGroup().getWeight().orElse(0);
    }

    public Group getLuckPermsGroup() {
        return this.group;
    }

    @Override
    public String getArticle()
    {
        return this.article;
    }

    @Override
    public String getName()
    {
        return name;
    }

    @Override
    public String getTag()
    {
        return tag;
    }

    @Override
    public String getAbbr()
    {
        return abbr;
    }

    @Override
    public String getPlural()
    {
        return plural;
    }

    @Override
    public ChatColor getColor()
    {
        return color;
    }

    @Override
    public org.bukkit.ChatColor getTeamColor()
    {
        return teamColor;
    }

    @Override
    public String getColoredName()
    {
        return color + name;
    }

    @Override
    public Component getColoredTag()
    {
        return coloredTag;
    }

    @Override
    public String getColoredLoginMessage()
    {
        return article + ' ' + color + name;
    }

    @Override
    public boolean hasTeam()
    {
        return hasTeam;
    }

    @Override
    public boolean hasDefaultLoginMessage()
    {
        return hasDefaultLoginMessage;
    }
}
